#include <PropWare/serial/i2c/i2cmaster.h>
#include <unistd.h>
//#include <stdbool.h>

//adapted from https://github.com/lucasmaziero/LiquidCrystal_I2C

// commands
#define LCD_CLEARDISPLAY 0x01
#define LCD_RETURNHOME 0x02
#define LCD_ENTRYMODESET 0x04
#define LCD_DISPLAYCONTROL 0x08
#define LCD_CURSORSHIFT 0x10
#define LCD_FUNCTIONSET 0x20
#define LCD_SETCGRAMADDR 0x40
#define LCD_SETDDRAMADDR 0x80

// flags for display entry mode
#define LCD_ENTRYRIGHT 0x00
#define LCD_ENTRYLEFT 0x02
#define LCD_ENTRYSHIFTINCREMENT 0x01
#define LCD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define LCD_DISPLAYON 0x04
#define LCD_DISPLAYOFF 0x00
#define LCD_CURSORON 0x02
#define LCD_CURSOROFF 0x00
#define LCD_BLINKON 0x01
#define LCD_BLINKOFF 0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 0x08
#define LCD_CURSORMOVE 0x00
#define LCD_MOVERIGHT 0x04
#define LCD_MOVELEFT 0x00

// flags for function set
#define LCD_8BITMODE 0x10
#define LCD_4BITMODE 0x00
#define LCD_2LINE 0x08
#define LCD_1LINE 0x00
#define LCD_5x10DOTS 0x04
#define LCD_5x8DOTS 0x00

// flags for backlight control
#define LCD_BACKLIGHT 0x08
#define LCD_NOBACKLIGHT 0x00

#define En 0b00000100  // Enable bit
#define Rw 0b00000010  // Read/Write bit
#define Rs 0b00000001  // Register select bit

class LiquidCrystal_I2C {
    public:
        LiquidCrystal_I2C (PropWare::I2CMaster *i2cbus,
                           uint8_t lcd_addr,
                           uint8_t lcd_cols,
                           uint8_t lcd_rows,
                           uint8_t charsize = LCD_5x8DOTS);

    public:
        int begin ();

        int clear ();

        int home ();

        int noDisplay ();

        int display ();

        int noBlink ();

        int blink ();

        int noCursor ();

        int cursor ();

        int scrollDisplayLeft ();

        int scrollDisplayRight ();

        int leftToRight ();

        int rightToLeft ();

        int noBacklight ();

        int backlight ();

        bool getBacklight ();

        int autoscroll ();

        int noAutoscroll ();

        int createChar (uint8_t location, uint8_t charmap[]);

        int setCursor (uint8_t col, uint8_t row);

        int write (uint8_t value);

        int command (uint8_t value);

        int blink_on ();

        int blink_off ();

        int cursor_on ();

        int cursor_off ();

        int setBacklight (bool new_val);

        int load_custom_character (uint8_t char_num, uint8_t *rows);

        int printstr (const char[]);

    private:
        uint8_t             _addr;
        uint8_t             _displayfunction;
        uint8_t             _displaycontrol;
        uint8_t             _displaymode;
        uint8_t             _cols;
        uint8_t             _rows;
        uint8_t             _charsize;
        uint8_t             _backlightval;
        PropWare::I2CMaster *_i2cbus;
    private:
        int send (uint8_t value, uint8_t mode);

        int write4bits (uint8_t value);

        int expanderWrite (uint8_t _data);

        int pulseEnable (uint8_t _data);

    private:
        static inline void delay (unsigned long millisec) {
            waitcnt(MILLISECOND * millisec + CNT);
        }

        static inline void delayMicro (unsigned long micro) {
            if (micro < 100)
                waitcnt(MICROSECOND * 100 + CNT);
            else
                waitcnt(MICROSECOND * micro + CNT);
        }
};
