#ifndef YM2612_H_
#define YM2612_H_

#include <PropWare/PropWare.h>
#include <PropWare/gpio/pin.h>
#include <PropWare/gpio/simpleport.h>

//#include <propeller.h>
//#include <simpletools.h>

// How long to hold the WR and CS lines when writing data.
// The datasheet indicates this should be *at least* 200nS, but it appears
// that a setting of (200) ~2,000uS ensures that the chip always gets our messages.
#define YM_WRITE_WAIT 11 // * 10uS

// The maximum about of uC cycles to wait for the YM2612 to be ready to receive data.
// This is around 10,000uS.
#define YM_MAX_WAIT_CYCLES 10000

// According to bus timing documents, we must hold the line low for at least
// 192 cycles. At 8hmz, that's 24 microseconds.
#define YM_RESET_WAIT 24 // uS

class YM2612
{
public:
	//YM2612(Bus * bus, uint8_t CS, uint8_t RD, uint8_t WR, uint8_t IC, uint8_t A0, uint8_t A1);
	YM2612(
		PropWare::SimplePort *bus,
		PropWare::Pin::Mask CS,
		PropWare::Pin::Mask RD,
		PropWare::Pin::Mask WR,
		PropWare::Pin::Mask IC,
		PropWare::Pin::Mask A0,
		PropWare::Pin::Mask A1,
		PropWare::Pin::Mask RDY
	);
    void Reset();
    void Send(uint8_t addr, uint8_t data, bool setA1 = false);
	void SendPacked(uint16_t value, bool setA1 = false); //packed format: 0xFF00 - Addr, 0xFF - Data
	void WaitReady(); //Wait until ready.
public:
	static inline void sleep37_5(unsigned int n);
private:
	PropWare::Pin _CS;
	PropWare::Pin _RD;
	PropWare::Pin _WR;
	PropWare::Pin _IC;
	PropWare::Pin _A0;
	PropWare::Pin _A1;
	PropWare::Pin _RDY;
	PropWare::SimplePort *_bus;
};
#endif

//Notes
//YM_CS = PB3
//YM_RD = PA15
//YM_WR = PA12
//YM_A0 = PA11
//YM_A1 = PB0
//YM_IC = PB4 
